/*******************************************************************************
 * Copyright (c) 2007  NTT DATA CORPORATION
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Version: 1.0.0 - 2007/06/15
 *          initial API and implementation
 *******************************************************************************/
package jp.sourceforge.tomoyo.core.ui.propertysheet.domain;

import jp.sourceforge.tomoyo.core.local.model.PolicyCacheManager;
import jp.sourceforge.tomoyo.core.local.model.PolicyElement;
import jp.sourceforge.tomoyo.core.local.model.AbstractPolicyModel;
import jp.sourceforge.tomoyo.core.local.model.domain.AccessPermission;
import jp.sourceforge.tomoyo.core.local.model.domain.Domain;
import jp.sourceforge.tomoyo.core.ui.propertysheet.ElementPropertySource;

public class DomainPropertySource extends ElementPropertySource {
	
	public DomainPropertySource(PolicyElement element) {
		super(element);
	}

	private static final String PROPERTY_ID_DOMAIN = "p.domain"; //$NON-NLS-1$
	private static final String PROPERTY_ID_DOMAIN_DEPTH = "p.domain_depth"; //$NON-NLS-1$
	private static final String PROPERTY_ID_PROCESS = "p.process"; //$NON-NLS-1$
	private static final String PROPERTY_ID_PROFILE = "p.profile"; //$NON-NLS-1$
	private static final String PROPERTY_ID_IS_INITIALIZED_TARGET = "p.is_initialized_target"; //$NON-NLS-1$
	private static final String PROPERTY_ID_IS_INITIALIZED_SOURCE = "p.is_initialized_source"; //$NON-NLS-1$
	private static final String PROPERTY_ID_KEEP_DOMAIN = "p.keep_domain"; //$NON-NLS-1$
	private static final String PROPERTY_ID_RESOURCE_COUNT = "p.resource_count"; //$NON-NLS-1$
	private static final String PROPERTY_ID_RESOURCE_REACHABLE = "p.reachable"; //$NON-NLS-1$
	private static final String PROPERTY_ID_DESCENDENT_DOMAIN_COUNT = "p.descendent_domain_count"; //$NON-NLS-1$

	protected String[] getPropertyIDs() {
		return new String[] {
				PROPERTY_ID_DOMAIN,
				PROPERTY_ID_DOMAIN_DEPTH,
				PROPERTY_ID_PROCESS,
				PROPERTY_ID_PROFILE,
				PROPERTY_ID_IS_INITIALIZED_TARGET,
				PROPERTY_ID_IS_INITIALIZED_SOURCE,
				PROPERTY_ID_KEEP_DOMAIN,
				PROPERTY_ID_RESOURCE_COUNT,
				PROPERTY_ID_RESOURCE_REACHABLE,
				PROPERTY_ID_DESCENDENT_DOMAIN_COUNT,
		};
	}

	protected String getPropertyName(String propertyID) {
		if (propertyID.equals(PROPERTY_ID_DOMAIN))
			return Messages.DomainPropertySource_Domain;
		if (propertyID.equals(PROPERTY_ID_DOMAIN_DEPTH))
			return Messages.DomainPropertySource_DomainDepth;
		if (propertyID.equals(PROPERTY_ID_PROCESS))
			return Messages.DomainPropertySource_ProcessName;
		if (propertyID.equals(PROPERTY_ID_PROFILE))
			return Messages.DomainPropertySource_Profile;
		if (propertyID.equals(PROPERTY_ID_IS_INITIALIZED_TARGET))
			return Messages.DomainPropertySource_IsInitializedTarget;
		if (propertyID.equals(PROPERTY_ID_IS_INITIALIZED_SOURCE))
			return Messages.DomainPropertySource_IsInitializedSource;
		if (propertyID.equals(PROPERTY_ID_KEEP_DOMAIN))
			return Messages.DomainPropertySource_KeepDomain;
		if (propertyID.equals(PROPERTY_ID_RESOURCE_COUNT))
			return Messages.DomainPropertySource_ResourceCount;
		if (propertyID.equals(PROPERTY_ID_RESOURCE_REACHABLE))
			return Messages.DomainPropertySource_Reachble;
		if (propertyID.equals(PROPERTY_ID_DESCENDENT_DOMAIN_COUNT))
			return Messages.DomainPropertySource_ChildrenCount2Bottom;
		return null;
	}

	public Object getPropertyValue(Object id) {
		if (id == null)
			return null;
		Domain domain = (Domain)element;
		if (id.equals(PROPERTY_ID_DOMAIN))
			return domain.getText();
		if (id.equals(PROPERTY_ID_DOMAIN_DEPTH))
			return Integer.valueOf(domain.getDepth());
		if (id.equals(PROPERTY_ID_PROCESS))
			return domain.getProcessName();
		if (id.equals(PROPERTY_ID_PROFILE)) {
			if (domain.getProfile() != null)
				return domain.getProfile().getLongName();
		}
		if (id.equals(PROPERTY_ID_IS_INITIALIZED_TARGET))
			return domain.isInitializerTarget() ? Messages.DomainPropertySource_YES : Messages.DomainPropertySource_NO;
		if (id.equals(PROPERTY_ID_IS_INITIALIZED_SOURCE))
			return domain.isInitializerSource() ? Messages.DomainPropertySource_YES : Messages.DomainPropertySource_NO;
		if (id.equals(PROPERTY_ID_KEEP_DOMAIN))
			return domain.isKeepDomain() ? Messages.DomainPropertySource_YES : Messages.DomainPropertySource_NO;
		if (id.equals(PROPERTY_ID_RESOURCE_COUNT)) {
			AbstractPolicyModel document = (AbstractPolicyModel)domain.getParent();
			if (document == null)
				return null;
			int count = PolicyCacheManager.getInstance().count(document.getProject(), domain, AccessPermission.class, false);
			return Integer.valueOf(count);
		}
		if (id.equals(PROPERTY_ID_RESOURCE_REACHABLE)) {
			if (domain.isReachable()) {
				return Messages.DomainPropertySource_YES;
			} else {
				switch (domain.getUnreachableReason()) {
					case Domain.UNREACHABLE_REASON_NO_PARENTS:
						return Messages.DomainPropertySource_NO + "(" + "No parerents" + ")"; //$NON-NLS-1$ //$NON-NLS-3$
					case Domain.UNREACHABLE_REASON_PARENT_INITIALIZED:
						return Messages.DomainPropertySource_NO + "(" + "Parent domain is initialized" + ")"; //$NON-NLS-1$ //$NON-NLS-3$
					case Domain.UNREACHABLE_REASON_PARENT_KEPTDOMAIN:
						return Messages.DomainPropertySource_NO + "(" + "Parent domain is keptdomain" + ")"; //$NON-NLS-1$ //$NON-NLS-3$
					case Domain.UNREACHABLE_REASON_PARENT_UNREACHABLE:
						return Messages.DomainPropertySource_NO + "(" + "Parent domain is unreachable" + ")"; //$NON-NLS-1$ //$NON-NLS-3$
					case Domain.UNREACHABLE_REASON_INITIALIZED:
						return Messages.DomainPropertySource_NO + "(" + "Initialised" + ")"; //$NON-NLS-1$ //$NON-NLS-3$
				}
			}
		}
		if (id.equals(PROPERTY_ID_DESCENDENT_DOMAIN_COUNT)) {
			AbstractPolicyModel document = (AbstractPolicyModel)domain.getParent();
			if (document == null)
				return null;
			return Integer.valueOf(PolicyCacheManager.getInstance().listDecendantDomainCount(document.getProject(), domain));
		}
			
		return null;
	}

}
